/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Genius Video Wonder Pro II V2/FM remote control driver.
 Valid for:   FlyVideo II/98 series TV-cards         

 Prototype (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// BTTool.exe display for "Video Wonder Pro II V2/FM":
// GPIO_Dma_Ctl - 0000c004; (0000c007 when TV window open)
// GPIO_Out_En  - 00001800;
// GPIO_Data    - remarked column (for pressed - released
//                keys and loaded/unload4ed JTV).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 00f4e700 (when JTV-window is open)
// GPIO_Data    - 00f4ff00 (when JTV-window is closed)


KeyDef keys[]=
{
//                                         JTV is loaded         JTV is NOT loaded
// 1-st row keys                           Pressed    Released	 Pressed  Released
        { 0x00, "Key_Power", "Power", }, //00f4e600 - 00f4e700 / f4fe00 - f4ff00
// No key
        { 0x80, "Key_MouseMode", "", },  //00f4e680 - 00f4e700 / f4fe80 - f4ff00
        { 0x40, "Key_TVFullScreen", "",},//00f4e640 - 00f4e700 / f4fe40 - f4ff00
// 2-nd row keys
        { 0x48, "Key_ChannelUp","ChannelUp",},//00f4e648 - 00f4e700	/ f4fe48 - ...
        { 0xc0, "Key_1", "1", },              //00f4e6c0 - 00f4e700	/ ...
        { 0x20, "Key_2", "2", },              //00f4e620 - 00f4e700
        { 0xa0, "Key_3", "3", },              //00f4e6a0 - 00f4e700
// ...
        { 0xc8, "Key_ChannelDown","ChannelDown",},//00f4e6c8 - 00f4e700
        { 0xe0, "Key_4", "4", },                  //00f4e6e0 - 00f4e700
        { 0x10, "Key_5", "5", },                  //00f4e610 - 00f4e700
        { 0x90, "Key_6", "6", },                  //00f4e690 - 00f4e700

        { 0x28, "Key_VolumeUp","VolumeUp",},  //00f4e628 - 00f4e700
        { 0xd0, "Key_7", "7", },              //00f4e6d0 - 00f4e700
        { 0x30, "Key_8", "8", },              //00f4e630 - 00f4e700
        { 0xb0, "Key_9", "9", },              //00f4e6b0 - 00f4e700

        { 0xe8, "Key_VolumeDown","VolumeDown", },//00f4e6e8 - 00f4e700
        { 0xf8, "Key_Cancel", "", },             //00f4e6f8 - 00f4e700
        { 0xf0, "Key_0", "0", },                 //00f4e6f0 - 00f4e700
        { 0x08, "Key_Enter", "Enter", },         //00f4e608 - 00f4e700

        { 0x60, "Key_Recall","Recall",},//00f4e660 - 00f4e700
        { 0x70, "Key_AirCableTV", "", },//00f4e670 - 00f4e700
        { 0x88, "Key_Video", "", },     //00f4e688 - 00f4e700
        { 0x50, "Key_Inc", "", },       //00f4e650 - 00f4e700

        { 0x48, "Key_Left", "Left", },  //00f4e648 - 00f4e700
        { 0xc8, "Key_Empty", "", },     //00f4e6c8 - 00f4e700
        { 0xa8, "Key_Audio", "Right", },//00f4e6a8 - 00f4e700
        { 0x68, "Key_Dec", "", },       //00f4e668 - 00f4e700

        { 0xb8, "Key_Window", "", },   //00f4e6b8 - 00f4e700
        { 0x98, "Key_Down", "", },     //00f4e698 - 00f4e700
        { 0x58, "Key_Esc", "", },      //00f4e658 - 00f4e700
// No key

        { 0x38, "Key_TabRight", "", }, //00f4e638 - 00f4e700
        { 0x78, "Key_TabLeft", "", },  //00f4e678 - 00f4e700
        { 0xd8, "Key_Mute", "", },     //00f4e6d8 - 00f4e700
        { 0x18, "Key_AVSource", "", }, //00f4e618 - 00f4e700
};


typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=200;	// default first delay for a key
INT repeat_delay=20;	// default repeat delay for a key


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
                log("> Initializing Genius Video Wonder Pro II remote module.    \n");
                log("> Copyright (c) 2000 by Ivan Kolin, <ivkolin@mail.ru>.\n");
                log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		};



        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // initial repeat delay

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0xc007);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00001800));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0xf4e700);
                   
				   
                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
                         DWORD key_flag=0;

                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 						 
                         value = (gpio_data &0x000000ff);
                         key_flag = ((gpio_data &0x0000ff00)>>8);

                         if ((key_flag == 0xe6) | (key_flag == 0xfe)) // any key is pressed
                         {	 
                             if (last_code == value)
                             {     // auto repeat for any pressed key
                                   AddStroke(last_code);
                            	   key_pause = repeat_delay;
                             } 
                             else
                             {	   // first run for any pressed key 
                                   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = first_delay;
                             }
                         }
                         else  // no one key is pressed
                         {	  
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }
							   
							   
                                key_pause = 20;
                         };
				
                    };

                log("> Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
